# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/nixtlats/',
                'doc_host': 'https://Nixtla.github.io',
                'git_url': 'https://github.com/Nixtla/nixtla/',
                'lib_path': 'nixtlats'},
  'syms': { 'nixtlats.client': {},
            'nixtlats.core.api_error': {},
            'nixtlats.core.client_wrapper': {},
            'nixtlats.core.datetime_utils': {},
            'nixtlats.core.jsonable_encoder': {},
            'nixtlats.core.remove_none_from_dict': {},
            'nixtlats.distributed.timegpt': { 'nixtlats.distributed.timegpt._DistributedTimeGPT': ( 'distributed.timegpt.html#_distributedtimegpt',
                                                                                                    'nixtlats/distributed/timegpt.py'),
                                              'nixtlats.distributed.timegpt._DistributedTimeGPT._forecast': ( 'distributed.timegpt.html#_distributedtimegpt._forecast',
                                                                                                              'nixtlats/distributed/timegpt.py'),
                                              'nixtlats.distributed.timegpt._DistributedTimeGPT._get_forecast_schema': ( 'distributed.timegpt.html#_distributedtimegpt._get_forecast_schema',
                                                                                                                         'nixtlats/distributed/timegpt.py'),
                                              'nixtlats.distributed.timegpt._DistributedTimeGPT._instantiate_timegpt': ( 'distributed.timegpt.html#_distributedtimegpt._instantiate_timegpt',
                                                                                                                         'nixtlats/distributed/timegpt.py'),
                                              'nixtlats.distributed.timegpt._DistributedTimeGPT.forecast': ( 'distributed.timegpt.html#_distributedtimegpt.forecast',
                                                                                                             'nixtlats/distributed/timegpt.py')},
            'nixtlats.errors.unprocessable_entity_error': {},
            'nixtlats.timegpt': { 'nixtlats.timegpt.TimeGPT': ('timegpt.html#timegpt', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt.TimeGPT.forecast': ('timegpt.html#timegpt.forecast', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT': ('timegpt.html#_timegpt', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT.__init__': ('timegpt.html#_timegpt.__init__', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._add_date_features': ( 'timegpt.html#_timegpt._add_date_features',
                                                                                    'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._compute_date_feature': ( 'timegpt.html#_timegpt._compute_date_feature',
                                                                                       'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._forecast': ('timegpt.html#_timegpt._forecast', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._get_model_params': ( 'timegpt.html#_timegpt._get_model_params',
                                                                                   'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._get_to_dict_args': ( 'timegpt.html#_timegpt._get_to_dict_args',
                                                                                   'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._hit_multi_series_endpoint': ( 'timegpt.html#_timegpt._hit_multi_series_endpoint',
                                                                                            'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._hit_multi_series_historic_endpoint': ( 'timegpt.html#_timegpt._hit_multi_series_historic_endpoint',
                                                                                                     'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._infer_freq': ('timegpt.html#_timegpt._infer_freq', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._make_future_dataframe': ( 'timegpt.html#_timegpt._make_future_dataframe',
                                                                                        'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._multi_series': ('timegpt.html#_timegpt._multi_series', 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._parse_response': ( 'timegpt.html#_timegpt._parse_response',
                                                                                 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._preprocess_dataframes': ( 'timegpt.html#_timegpt._preprocess_dataframes',
                                                                                        'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._resample_dataframe': ( 'timegpt.html#_timegpt._resample_dataframe',
                                                                                     'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._transform_dataframes': ( 'timegpt.html#_timegpt._transform_dataframes',
                                                                                       'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._validate_input_size': ( 'timegpt.html#_timegpt._validate_input_size',
                                                                                      'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._validate_inputs': ( 'timegpt.html#_timegpt._validate_inputs',
                                                                                  'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT._validate_outputs': ( 'timegpt.html#_timegpt._validate_outputs',
                                                                                   'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT.request_headers': ( 'timegpt.html#_timegpt.request_headers',
                                                                                 'nixtlats/timegpt.py'),
                                  'nixtlats.timegpt._TimeGPT.validate_token': ( 'timegpt.html#_timegpt.validate_token',
                                                                                'nixtlats/timegpt.py')},
            'nixtlats.types.http_validation_error': {},
            'nixtlats.types.multi_series_input': {},
            'nixtlats.types.single_series_forecast': {},
            'nixtlats.types.validation_error': {},
            'nixtlats.types.validation_error_loc_item': {}}}
